import java.lang.reflect.Modifier;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import javax.lang.model.element.AnnotationValue;
import javax.lang.model.element.Element;

final class StubBuilder {

	private static final String RETURN_TYPE = "returnType";
	private static final String PARAMETER_TYPES = "parameterTypes";

	private final StringBuilder contents;
	private boolean classNameSet;
	private String interfaceName;

	StubBuilder() {
		this.contents = new StringBuilder();
		this.classNameSet = false;
	}

	StubBuilder setClassName(final String interfaceName) {
		this.interfaceName = interfaceName;
		
		if (!this.classNameSet) {
			this.contents.append("public class ");
			this.contents.append(interfaceName).append("Impl");
			this.contents.append(" implements ");
			this.contents.append(interfaceName);
			this.contents.append(" {\n\n");

			this.classNameSet = true;
		}

		return this;
	}

	StubBuilder setMethod(final Element annotatedElement, final Map<String, Object> annotationElements) {

		final String accessModifier = Modifier.toString(Modifier.PUBLIC);
		final String returnType = Objects.isNull(annotationElements.get(RETURN_TYPE)) ? Void.class.getSimpleName().toLowerCase()
				: annotationElements.get(RETURN_TYPE).toString();

		@SuppressWarnings("unchecked")
		final List<String> parameterTypes = annotationElements.containsKey(PARAMETER_TYPES)
				? convertParameterTypes((List<? extends AnnotationValue>) annotationElements.get(PARAMETER_TYPES))
				: Collections.emptyList();

		this.contents.append("\t @Override\n");
		this.contents.append("\t ");
		this.contents.append(accessModifier);
		this.contents.append(" ");

		this.contents.append(returnType);

		this.contents.append(" ");
		this.contents.append(annotatedElement.getSimpleName().toString());
		this.contents.append("(");
		appendMethodParameters(parameterTypes);
		this.contents.append(")");

		if (returnType.contains("void")) {
			this.contents.append("\t {}\n\n");
		} else {
			this.contents.append("\t { return null; }\n\n");
		}

		return this;
	}

	String build() {
		this.contents.append("}");
		return this.contents.toString();
	}
	
	String getInterfaceName() {
		return this.interfaceName;
	}

	private void appendMethodParameters(final List<String> parameterTypes) {
		assert !Objects.isNull(parameterTypes);

		final String argPrefix = "arg";

		for (int i = 0; i < parameterTypes.size(); i++) {
			if (i > 0) {
				this.contents.append(", ");
			}

			this.contents.append(parameterTypes.get(i));
			this.contents.append(" ");
			this.contents.append(argPrefix);
			this.contents.append(i);
		}
	}

	private List<String> convertParameterTypes(final List<? extends AnnotationValue> values) {
		return values.stream().map(annotationValue -> {
			return annotationValue.getValue().toString();
		}).collect(Collectors.toList());
	}
}
